package com.fjhx.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.thread.ThreadUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.config.TaskPoolConfig;
import com.fjhx.entity.EmailInfo;
import com.fjhx.entity.EmailMailbox;
import com.fjhx.service.IAccountService;
import com.fjhx.service.IEmailInfoService;
import com.fjhx.service.IEmailMailboxService;
import com.fjhx.utils.EmailEngineUtil;
import com.fjhx.vo.BindingVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.concurrent.Executor;

@Service
public class AccountServiceImpl implements IAccountService {

    @Autowired
    private IEmailInfoService emailInfoService;

    @Autowired
    private IEmailMailboxService emailMailboxService;

    @Qualifier(TaskPoolConfig.taskExecutor)
    @Autowired
    private Executor executor;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public EmailInfo binding(BindingVo bindingVo) {

        String email = bindingVo.getEmail();

        EmailInfo emailInfo = emailInfoService.getOne(Wrappers.<EmailInfo>lambdaQuery().eq(EmailInfo::getEmail, email));
        // 如果存在，直接返回邮箱信息
        if (emailInfo != null) {
            return emailInfo;
        }

        // 添加账号
        EmailEngineUtil.createAccount(bindingVo);
        // 查询邮箱
        List<EmailMailbox> mailboxList = EmailEngineUtil.getMailboxList(email);
        // 保存账号数据
        emailInfo = saveEmailInfo(bindingVo);
        // 添加邮箱文件夹
        saveEmailMailbox(mailboxList, emailInfo.getId(), email);
        // 异步遍历文件夹下的所有邮件
        asyncReadEmail(email, mailboxList);
        return emailInfo;
    }

    /**
     * 添加邮件信息
     */
    private EmailInfo saveEmailInfo(BindingVo bindingVo) {
        EmailInfo emailInfo = BeanUtil.copyProperties(bindingVo, EmailInfo.class);
        emailInfoService.save(emailInfo);
        return emailInfo;
    }

    /**
     * 添加邮箱文件夹
     */
    private void saveEmailMailbox(List<EmailMailbox> mailboxList, Long emailInfoId, String email) {
        for (EmailMailbox emailMailbox : mailboxList) {
            emailMailbox.setEmailInfoId(emailInfoId);
            emailMailbox.setEmail(email);
            emailMailbox.setIsShow(true);
            emailMailbox.setInitialize(false);
        }
        emailMailboxService.saveBatch(mailboxList);
    }

    private void asyncReadEmail(String email, List<EmailMailbox> mailboxList) {
        executor.execute(() -> {

            ThreadUtil.sleep(5000);

            for (EmailMailbox emailMailbox : mailboxList) {
                EmailEngineUtil.getMessageList(email, emailMailbox.getPath());
            }

        });

    }

}
